PROGRAM tiller

!PROGRAM DESCRIPTION:
!THIS PROGRAM CALCULATES THE OPTIMAL TILLER PROFILE OF A GIVEN BOW SHAPE (SIDE-PROFILE AND TOP-PROFILE)
!FREE PARAMETERS ARE STRING-LENGHT,BOW-LENGHT,DRAW-LENGHT, BOW-CENTER AND ELASTICITY PARAMETERS OF THE GIVEN WOOD
!THE PROGRAMS OUTPUT CONTAINS ALSO THE OPTIMAL THICKNES PROFILE AND MAXIMUM POSSIBLE DRAW-WEIGHT OF THE HYPOTHETICAL BOW
!TOP AND SIDE PROFILES  CAN BE TAKEN OF THE BOW STAVE BEFORE TILLERING
!BOW PIXEL HAVE PIXEL VALUE ZERO (BLACK)
!FILES HAVE TO BE IN THE "PGM"-FORMAT, IMAGE DIMENSIONS OF BOTH FILES HAVE TO BE IN ACCORDANCE 
!OUTPUT DATA IS IN CARTESIAN COORDINATES WITH THE SAME UNITS AS INPUT UNITS
!PROPOSED STANDARD UNIT FOR STRESS IS NEWTON PER SQUARE CENTIMETER


IMPLICIT NONE

!PARAMETER SECTION, PLEASE SET:

INTEGER, PARAMETER::iun=20
INTEGER, DIMENSION(2)::end_pt                         !ENDING POINT OF BENDING SECTION, E.G. BEGINNING OF A STATIC RECURVE
DOUBLE PRECISION, PARAMETER:: max_tens=12050.      !MAXIMUM POSSIBLE DISTENSION STRESS (DISTENSION RESISTANCE) 
DOUBLE PRECISION, PARAMETER:: max_comp=12050.       !MAXIMUM POSSIBLE COMPRESSION STRESS (COMPRESSION RESISTANCE)
DOUBLE PRECISION, PARAMETER:: x_lnght_0=137.          !BOW LENGHT (GEOMETRICAL DISTANCE BETWENN BOTH STRING POINTS)
DOUBLE PRECISION, PARAMETER::x_lnght_l=68.            !BOW CENTER(GEOMETRICAL DISTANCE BETWEEN LEFT STRING POINT AND BOW CENTER)
DOUBLE PRECISION, PARAMETER:: E_mod=1.20507E6                !ELASTICITY MODULE OF THE GIVEN WOOD
DOUBLE PRECISION, PARAMETER:: draw_lnght=65.          !DRAW-LENGHT
DOUBLE PRECISION, PARAMETER:: s_lnght_0=135.5         !STRING-LENGHT
DOUBLE PRECISION, PARAMETER::bend_fract=0.75           !HOW MUCH OF THE LIMB BENDS (0.75 => OUTER 25% REMAIN STIFF) 
DOUBLE PRECISION:: a, recovery,max_draw_wght
DOUBLE PRECISION, PARAMETER:: init_wght=6000.       !PARAMETER THAT HAS TO BE ADJUSTED FOR ALGORITHM CONVERGENCE (APPROX. 25X EXPECTED DRAW WEIGHT)
DOUBLE PRECISION, PARAMETER::tune=2E-4           !TUNING FACTOR TO OPTIMIZE ALGORITHM PERFORMANCE
DOUBLE PRECISION, DIMENSION(2)::draw_wght, elem_lnght, lmb_lnght, x_m, y_m, x_lnght, s_lnght, s_tens, s_angle
DOUBLE PRECISION, ALLOCATABLE,DIMENSION(:,:,:)::pos
DOUBLE PRECISION, ALLOCATABLE,DIMENSION(:,:):: angle, angle_0, thick, width, curve, path_lnght
INTEGER, ALLOCATABLE, DIMENSION(:,:,:):: side, top
CHARACTER(LEN=128):: file_1,file_2, path
INTEGER, DIMENSION(2)::pixel, lines
INTEGER::i,j,pixel_0,pixel_l


file_1="profile_side.pgm"         !SIDE PROFILE FILENAME
file_2="profile_top.pgm"          !TOP PROFILE FILENAME 



!ENDING OF PARAMETER SECTION



CALL read_file(side, file_1,lines(1),pixel_0,x_lnght_0, x_lnght_l, pixel_l)  !READ IN SIDE PROFILE OF BOW
CALL read_file(top, file_2,lines(2), pixel_0,x_lnght_0,x_lnght_l,pixel_l)   !READ IN TOP PROFILE OF BOW



ALLOCATE(angle(pixel_0,2))
ALLOCATE(angle_0(pixel_0,2))
ALLOCATE(thick(pixel_0,2))
ALLOCATE(width(pixel_0,2))
ALLOCATE(pos(pixel_0,2,2))
ALLOCATE(curve(pixel_0,2))
ALLOCATE(path_lnght(pixel_0,2))

x_lnght(1)=x_lnght_l                  !PHYSICAL LIMB X-DIMENSION 
x_lnght(2)=x_lnght_0-x_lnght_l
pixel(1)=pixel_l                      !PIXEL LIMB X-DIMENSION  
pixel(2)=pixel_0-pixel_l+1
s_lnght=s_lnght_0/x_lnght_0*x_lnght


CALL init_angle(angle_0,side,curve,path_lnght, lines(1),pixel, pixel_0,x_lnght)
CALL init_width(top,width,path_lnght,lines(2), pixel,pixel_0,x_lnght_0, x_lnght)

end_pt=INT(bend_fract*Pixel)

elem_lnght(1)=path_lnght(pixel(1)-1,1)/DBLE(pixel(1))
elem_lnght(2)=path_lnght(pixel(2)-1,2)/DBLE(pixel(2))

s_angle=85.*3.1415926/180    !GUESS AN INITIAL STRING ANGLE TO DEFINE INITIAL FORCE ATTACKING POINT


DO j=1,2
   DO        !REPEAT UNTIL TIP POSITION AND FORCE ATTACKING POINT MATCH
      draw_wght(j)=init_wght
      pos(:,:,j)=0.
      angle(:,j)=0.
      x_m(j)=s_lnght(j)*SIN(s_angle(j))
      y_m(j)=-draw_lnght + s_lnght(j)*COS(s_angle(j))
      s_tens(j)=draw_wght(j)/(2*COS(s_angle(j)))

      DO             !BEGIN TO BEND WITH INCREASED DRAW-WEIGHT UNTIL LIMB TIP REACHES RADIUS OF STRING-LENGHT
         pos(:,:,j)=0.
         angle(:,j)=0. 
         DO i=2,end_pt(j)
            CALL bending(pos(:,:,j),x_m(j), y_m(j), thick(:,j),width(:,j), angle(:,j), angle_0(:,j), s_angle(j)&
                 , s_tens(j),  max_tens, max_comp, E_mod, elem_lnght(j),pixel_0,i)
         END DO
         CALL init_fixed_thick(thick(:,j),pixel_0, pixel(j),end_pt(j))
         DO i=end_pt(j)+1,pixel(j)-1
            CALL fixed_thick(pos(:,:,j), x_m(j), y_m(j), thick(:,j),width(:,j), angle(:,j), angle_0(:,j), s_angle(j)&
                 , s_tens(j), E_mod, elem_lnght(j),pixel_0,i)
         END DO

         IF (((draw_lnght+pos(pixel(j),2,j))**2+pos(pixel(j),1,j)**2-s_lnght(j)**2) .LT. 0.01) EXIT
         draw_wght(j)=draw_wght(j)-init_wght*tune
         s_tens(j)=draw_wght(j)/(2*COS(s_angle(j)))
      END DO
      s_angle(j)=ATAN(pos(pixel(j),1,j)/(draw_lnght+pos(pixel(j),2,j)))
      IF (((x_m(j)-pos(pixel(j),1,j))**2+(y_m(j)-pos(pixel(j),2,j))**2).LT. 0.01) EXIT
   END DO
END DO

recovery=s_tens(1)/s_tens(2)            !MATCH LEFT AND RIGHT HAND SIDE STRING-TENSION
thick(:,2)=thick(:,2)*SQRT(recovery)
max_draw_wght=MIN(draw_wght(1),draw_wght(2))



OPEN(iun, FILE="data.txt", STATUS='OLD')
    WRITE(IUN, FMT='(A11,3X,F9.5)') "draw weight", max_draw_wght
    WRITE(iun, FMT='(A10,10X,A2,10X,A9)')'x','y','thickness'
    DO i=1, pixel(1)-1
       WRITE(iun,FMT='(F9.5, 6X, F9.5, 6X, F15.5)')-pos(pixel(1)+1-i,1,1), pos(pixel(1)+1-i,2,1), thick(pixel(1)+1-i,1)
    END DO
    DO i=2, pixel(2)
       WRITE(iun,FMT='(F9.5, 6X, F9.5, 6X, F15.5 )')pos(i,1,2), pos(i,2,2), thick(i,2)
    END DO
   CLOSE(iun)
CONTAINS

SUBROUTINE read_file(data,file, lines,pixel, x_lnght, x_lnght_l, pixel_l)      !READS IN PGM FILES
    IMPLICIT NONE
    INTEGER, PARAMETER::iun=20
    INTEGER, INTENT(INOUT)::lines, pixel, pixel_l
    DOUBLE PRECISION, INTENT(IN)::x_lnght, x_lnght_l
    INTEGER::i,j
    INTEGER, ALLOCATABLE,DIMENSION(:,:,:)::data
    CHARACTER(len=128)::file,header1, header2
    OPEN(iun, FILE=TRIM(file), STATUS='OLD')
    READ(iun,*)header1
    READ(iun,*)header2
    READ(iun, FMT='(2(I4))')pixel, lines
    READ(iun,*)header1
    ALLOCATE(data(lines,pixel,2))
    pixel_l=INT((DBLE(x_lnght_l)/DBLE(x_lnght))*pixel)
    DO j=1,lines
       DO i=1, pixel_l
          READ(iun,FMT='(I3)')data(j,pixel_l+1-i,1)
       END DO
       data(j,1,2)=data(j,1,1)
       DO i=pixel_l+1,pixel
          READ(iun, FMT='(I3)')data(j,1+i-pixel_l,2)
       END DO
    END DO
    CLOSE(iun)
END SUBROUTINE


SUBROUTINE init_angle(angle, data,curve,path_lnght, lines,m,pixel_0,x_lnght)        !INITIALISES UNSTRUNG SIDE PROFILE  (ANGLE COORDINATE)
  IMPLICIT NONE
  DOUBLE PRECISION, INTENT (INOUT),DIMENSION(pixel_0,2)::angle, curve, path_lnght
  DOUBLE PRECISION, DIMENSION(2),INTENT(IN)::x_lnght !PHYSICAL LIMB LENGHT
  INTEGER, DIMENSION(lines,pixel_0,2), INTENT(IN)::data
  INTEGER, INTENT(IN)::lines, pixel_0
  INTEGER, DIMENSION(2), INTENT(IN)::m
  INTEGER:: i,j,l
  curve=0
  DO l=1,2
     DO j=1,lines
        DO i=1,m(l)
           IF(DBLE(data(j,i,l)).EQ.0.) THEN 
              a=j
              IF(curve(i,l).EQ.0) THEN
                 curve(i,l)=-DBLE(a)*x_lnght(l)/(DBLE(m(l)-1))
              END IF
           END IF
        END DO
     END DO
     DO i=1,m(l)-1
       angle(i,l)=ATAN((curve(i+1,l)-curve(i,l))/(x_lnght(l)/(DBLE(m(l)-1))))
     END DO
     CALL calc_path_lnght(path_lnght(:,l),curve(:,l), pixel_0, pixel(l), x_lnght(l))
     CALL convert_cart_to_nat(angle(:,l),path_lnght(:,l),pixel_0,m(l), x_lnght(l)) 
  END DO
END SUBROUTINE 


SUBROUTINE init_width (profile,width,path_lnght,lines, pixel, pixel_0,x_lnght_0, x_lnght) !INITIALISES LIMB-WIDTH
  IMPLICIT NONE
  DOUBLE PRECISION, DIMENSION(pixel_0,2), INTENT(INOUT)::width
  DOUBLE PRECISION, DIMENSION(pixel_0,2), INTENT(IN)::path_lnght
  DOUBLE PRECISION, INTENT(IN)::x_lnght_0
  DOUBLE PRECISION, DIMENSION(2),INTENT(IN)::x_lnght
  INTEGER, DIMENSION(lines,pixel_0,2), INTENT(INOUT)::profile
  INTEGER::i,j,k
  INTEGER, DIMENSION(2),INTENT(IN)::pixel
  INTEGER, INTENT(IN):: lines, pixel_0
  width=0
  DO k=1,2
     DO j=1,pixel(k)
        DO i=1,lines
           IF(profile(i,j,k).EQ.0) THEN            
              width(j,k)=width(j,k)+x_lnght_0/DBLE(pixel_0)
           END IF
        END DO
     END DO
     CALL convert_cart_to_nat(width(:,k),path_lnght(:,k), pixel_0,pixel(k),x_lnght(k))
  END DO
END SUBROUTINE

SUBROUTINE bending (pos, x_m, y_m, thick,width, angle, angle_0, s_angle, s_tens,  max_tens, max_comp, E_mod, elem_lnght,m,i) 
!APPLY PHYSICS TO BENDING LIMB SECTION
     IMPLICIT NONE
     DOUBLE PRECISION, DIMENSION(m), INTENT(INOUT)::thick, angle, angle_0, width
     DOUBLE PRECISION, DIMENSION(m,2), INTENT(INOUT):: pos
     DOUBLE PRECISION, INTENT(IN)::x_m, y_m,max_tens,max_comp,E_mod,elem_lnght, s_tens, s_angle
     INTEGER, INTENT(IN)::m,i
     DOUBLE PRECISION:: torque,phi
      torque=SQRT((x_m-pos(i,1))**2+(y_m-pos(i,2))**2)*&
      s_tens*SIN(3.1415926/2.0 - s_angle +ATAN((pos(i,2)-y_m)/(x_m-pos(i,1))))
      thick(i)=SQRT(torque/(width(i)*MIN(max_tens,max_comp))*6)
      phi=ASIN((2*MIN(max_tens,max_comp)*elem_lnght/(E_mod*thick(i))))
      angle(i+1)=angle(i)-phi+angle_0(i+1)-angle_0(i)
      pos(i+1,1)=pos(i,1)+elem_lnght*COS(angle(i))
      pos(i+1,2)=pos(i,2)+elem_lnght*SIN(angle(i))
END SUBROUTINE

SUBROUTINE fixed_thick(pos, x_m, y_m, thick,width, angle, angle_0, s_angle, s_tens, E_mod, elem_lnght,m,i) 
!APPLY PHYSICS TO STIFF LIMB SECTION
     DOUBLE PRECISION, DIMENSION(m), INTENT(INOUT)::thick, angle, angle_0, width
     DOUBLE PRECISION, DIMENSION(m,2), INTENT(INOUT):: pos
     DOUBLE PRECISION, INTENT(IN)::x_m, y_m,E_mod,elem_lnght, s_tens, s_angle
     INTEGER, INTENT(IN)::m,i
     DOUBLE PRECISION:: torque,phi, max_tens

     torque=SQRT((x_m-pos(i,1))**2+(y_m-pos(i,2))**2)*&
     s_tens*SIN(3.1415926/2.0 - s_angle +ATAN((pos(i,2)-y_m)/(x_m-pos(i,1))))
     max_tens=torque/(thick(i)**2+width(i)*3)
     phi=ASIN(2*MIN(max_tens, max_comp)*elem_lnght/(E_mod*thick(i)))
     angle(i+1)=angle(i)-phi+angle_0(i+1)-angle_0(i)
     pos(i+1,1)=pos(i,1)+elem_lnght*COS(angle(i))
     pos(i+1,2)=pos(i,2)+elem_lnght*SIN(angle(i))
END SUBROUTINE

SUBROUTINE init_fixed_thick(thick, m, pixel, end_pt)          !DEFINE THE STIFF LIMB SECTION WITH SMOOTH TRANSITION
  IMPLICIT NONE
  DOUBLE PRECISION, DIMENSION(m), INTENT(INOUT)::thick
  DOUBLE PRECISION::thick_0
  INTEGER, INTENT(IN)::m,end_pt, pixel
  INTEGER::i
  thick_0=thick(end_pt)
  DO i=end_pt+1, end_pt+20
     thick(i)=thick_0+0.4*thick_0/20.*DBLE(i-end_pt)
  END DO

  DO i=end_pt+21,pixel
     thick(i)=thick(i-1)
  END DO
END SUBROUTINE init_fixed_thick

SUBROUTINE calc_path_lnght(path_lnght,curve, pixel_0, pixel, x_lnght) !CALCULATES THE NATURAL COORDINATES (PATHLENGHT)
    IMPLICIT NONE
    DOUBLE PRECISION, DIMENSION(pixel_0), INTENT(INOUT)::path_lnght
    DOUBLE PRECISION, DIMENSION(pixel_0), INTENT(IN)::curve
    DOUBLE PRECISION, INTENT(IN)::x_lnght
    INTEGER, INTENT(IN)::pixel_0, pixel
    INTEGER:: i
    path_lnght(1)=SQRT((curve(2)-curve(1))**2+(x_lnght/(DBLE(pixel)))**2)
    DO i=2,pixel-1
       path_lnght(i)=path_lnght(i-1)+SQRT((curve(i+1)-curve(i))**2+(x_lnght/(DBLE(pixel)-1))**2)
   END DO
END SUBROUTINE


SUBROUTINE convert_cart_to_nat(data,path_lnght,pixel_0, pixel, x_lnght) !CONVERTS CARTESIAN INTO NATURAL COORDINATES
  IMPLICIT NONE
  DOUBLE PRECISION, DIMENSION(pixel_0), INTENT(INOUT)::data
  DOUBLE PRECISION, DIMENSION(pixel_0), INTENT(IN):: path_lnght 
  DOUBLE PRECISION, DIMENSION(pixel_0):: temp,temp2
  DOUBLE PRECISION, INTENT(IN)::x_lnght
  DOUBLE PRECISION:: bow_lnght
  INTEGER, INTENT(IN)::pixel, pixel_0
  INTEGER:: i,j,k
  temp=data
  temp2=path_lnght
  temp2=temp2/temp2(pixel-1)*DBLE(pixel)
  DO i=1,pixel-1
       DO j=1,pixel-1
          IF(INT(temp2(j)).EQ.i) THEN
             k=j
          ENDIF
       END DO
       data(i)=temp(k)
  END DO
END SUBROUTINE

END PROGRAM tiller
